#include "audio.h"
#include "canvas.h"
#include <process.h>
#include <stdio.h>

static int recordCallback( void *inputBuffer, void *outputBuffer,
                           unsigned long framesPerBuffer,
                           PaTimestamp outTime, void *userData )
{
  SAMPLE *rptr = (SAMPLE*)inputBuffer;
  Main   *base = (Main*)userData;
  double a;
 
  if( inputBuffer != NULL )
  {
    for(unsigned long  i=0; i<framesPerBuffer; i++ )
    {
 
      a = (double)(unsigned char)(*rptr++);
      a-= 128.0;

      base->setSignal( a , SIGNAL_MICROPHONE );  /*mono signal & 8bit = one byte per frame */
    }
  }

  return Audio::finalize;
}

int Audio::finalize               = 0;
int Audio::active                 = 0;
PortAudioStream *Audio::stream    = 0L;

Audio::Audio(Main *base_) : base(base_) 
{

  PaError err = Pa_Initialize();
  if( err != paNoError ) goto error;

  err = Pa_OpenStream(
            &stream,
            Pa_GetDefaultInputDeviceID(),
            NUM_CHANNELS,               /* stereo input */
            PA_SAMPLE_TYPE,
            NULL,
            paNoDevice,
            0,
            PA_SAMPLE_TYPE,
            NULL,
            SAMPLE_RATE,
            BUFFERSIZE,             /* frames per buffer */
            0,               /* number of buffers, if zero then use default minimum */
            0, //paDitherOff,    /* flags */
            recordCallback,
            (void*)base );
  if( err == paNoError ) return;

error:
    Pa_Terminate();
}

Audio::~Audio()
{
  Pa_Terminate();
}

void audiothread(void *)
{
  PaError err = Pa_StartStream( Audio::stream );
  if( err != paNoError ) return;

  while ( Pa_StreamActive( Audio::stream ) ) Pa_Sleep(500L);
  
  err = Pa_CloseStream( Audio::stream );
  if( err != paNoError ) goto error;
  
error:
  if (paNoError != err) Pa_Terminate();
  Audio::active = 0;
  _endthread();
}

void Audio::start()
{
  _beginthread( audiothread, 0 , 0 );
  active = 1;
}

void Audio::end()
{
  finalize = 1;
  Pa_Sleep(1000);  //make sure the thread is finished
}