#include <qdatetime.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qkeycode.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qimage.h>

#include "canvas.h"

#include <stdlib.h>
#include <math.h>
#include <stdio.h>

class CanvasData : public QCanvasLine
{
  int type_id;
public:
  CanvasData(QCanvas *canvas_) : QCanvasLine(canvas_) {}
  virtual int rtti () const { return type_id; }
  void        setTypeId(int i,int octave) { type_id = i + RTTI_DATA+RTTI_DATA*octave; }
};

ViewTimer::ViewTimer(QCanvas* canvas,Main *main_) :
    QCanvasSprite(0,canvas),base(main_)
{
    setAnimated(TRUE);
}

void ViewTimer::advance(int stage)
{
  //heart of this program:
  //1. if input is NOT microphone, create a test signal
  //2. update all line-segments of our display (they get redrawn automaticaly)
  double        a;
  
  QCanvasItemList           list  = canvas()->allItems();
  QCanvasItemList::Iterator it    = list.begin();
  CanvasData               *line  = 0L;
  Wavelets                 *wavelets=base->getWavelets();
  register                  delta = (int)ceil( (double)canvas()->width() / (double)DATAPOINTS );
  register                  h2    = canvas()->height()>>1;
  register                  h4    = h2>>1;
  register                  n,w;
  register                  x;

  //if no valid signal source is selected, use a testsignal..
  switch (base->selectedSource())
  {  
    case SIGNAL_SIN:    
      a = sin(base->getTime()*0.1);    
      base->setSignal( a , SIGNAL_SIN);
      break;
    case SIGNAL_SQUARE: 
      a = (base->getTime()&0x10)?1.0:0.0; 
      base->setSignal( a , SIGNAL_SQUARE);
      break;
  }
  

  //update linesegments.  redraw is automatic by Qt
  for (; it != list.end(); ++it) {
	  if ( *it && (*it)->rtti()>=RTTI_DATA )
      line =  dynamic_cast<CanvasData *>(*it);
      n    =  line->rtti() - RTTI_DATA;
      w    =  n/RTTI_DATA;
      n   %=  RTTI_DATA;
      x    =  n*delta;

      if (line) {
        if (n!=DATAPOINTS-1) {
          line->setPoints(  
            x,          h2 + h4*wavelets->get(n,  base->selectedS()?0:1,w,base->selectedMaxLock() ),
            (x+delta),  h2 + h4*wavelets->get(n+1,base->selectedS()?0:1,w,base->selectedMaxLock() )
            );
        }
      }
  }
}



WaveletView::WaveletView(
	QCanvas& c, QWidget* parent,
	const char* name, WFlags f) :
    QCanvasView(&c,parent,name,f)
{
}

void WaveletView::clear()
{
  QCanvasItemList list = canvas()->allItems();
  QCanvasItemList::Iterator it = list.begin();
  for (; it != list.end(); ++it) {
	  if ( *it && !dynamic_cast<ViewTimer*>(*it))  /*don't kill the timer ! */
	    delete *it;
  }
}

void Main::init()
{
  const QColor pens[MAX_OCTV]= {
    Qt::black, Qt::red, Qt::blue, Qt::darkGreen,Qt::yellow,Qt::cyan, Qt::magenta,Qt::darkGreen,Qt::darkYellow,Qt::white,
  };

  int          i,delta = (int)ceil( (double)canvas.width() / (double)DATAPOINTS );
  int          j,k;


  clear();

  for (k=0; k<MAX_OCTV; ++k) {
    for (i=0,j=0; selectedOctave(k) && j<DATAPOINTS-1; i+=delta,++j) {
      CanvasData* l = new CanvasData(&canvas);
      l->setPoints( (int)i,       canvas.height()>>1,
                    (int)(i+delta), canvas.height()>>1 );
      l->setPen( pens[k] );
      l->setAnimated( TRUE );
      l->setTypeId( j , k );
      l->show();	      
    }
  }

}

Main::Main(QCanvas& c, QWidget* parent, const char* name, WFlags f) :
    QMainWindow(parent,name,f),
    canvas(c),timer(&canvas,this),
    wavelets(DATAPOINTS),t(0), needMicrophone(1)
{
    char             s[20];
    QMenuBar* menu = menuBar();
    wview          = new WaveletView(canvas,this);

    audio = new Audio(this);
    audio->start();

    QPopupMenu* file = new QPopupMenu;
    file->insertItem("&Restart", this, SLOT(init()), CTRL+Key_F);
    file->insertSeparator();
    file->insertItem("E&xit", qApp, SLOT(quit()), CTRL+Key_Q);
    menu->insertItem("&File", file);

    edit = new QPopupMenu;
    menuid_s = edit->insertItem("&S-component");
    menuid_d = edit->insertItem("&D-component");
    edit->setItemChecked(menuid_s, TRUE);
    edit->insertSeparator();
    for (int i=0; i<MAX_OCTV; ++i) {
      sprintf(s,"Octave %d",i+1);
      menuid_o[i] = edit->insertItem(s);
    }
    edit->setItemChecked(menuid_o[0], TRUE);
    QObject::connect( edit, SIGNAL( activated(int)), this, SLOT( menuEdit(int) ) );
    menu->insertItem("&Watch", edit);

    sync = new QPopupMenu;
    menuid_max = sync->insertItem("Lock &scale");
    sync->setItemChecked(menuid_max, FALSE);
    sync->insertSeparator();
    for ( i=0; i<MAX_OCTV; ++i) {
      sprintf(s,"Octave %d",i+1);
      menuid_sync[i] = sync->insertItem(s);
    }
    sync->setItemChecked(menuid_sync[0], TRUE);
    QObject::connect( sync, SIGNAL( activated(int)), this, SLOT( menuSync(int) ) );
    menu->insertItem("&Synch", sync);

    signal = new QPopupMenu;
    menuid_source[0] = signal->insertItem("&Sinus");
    menuid_source[1] = signal->insertItem("S&quare");
    menuid_source[2] = signal->insertItem("&Microphone");
    signal->setItemChecked(menuid_source[2], TRUE);
    menu->insertItem("S&ignal", signal);
    QObject::connect( signal, SIGNAL( activated(int)), this, SLOT( menuSignal(int) ) );

    options = new QPopupMenu;
    menuid_dbf = options->insertItem("Double buffer", this, SLOT(toggleDoubleBuffer()));
    options->setItemChecked(menuid_dbf, TRUE);
    options->insertSeparator();
    options->insertItem("&Enlarge", this, SLOT(enlarge()), CTRL+Key_Plus);
    options->insertItem("Shr&ink", this, SLOT(shrink()), CTRL+Key_Minus);
    menu->insertItem("&Options",options);



    menu->insertSeparator();

    QPopupMenu* help = new QPopupMenu;
    help->insertItem("&Help", this, SLOT(help()), Key_F1);
    menu->insertItem("&Help",help);

    statusBar();

    setCentralWidget(wview);
    timer.setAnimated(TRUE);
    canvas.setBackgroundColor( Qt::blue.light( 190 ) );

    init();
}

Main::~Main()
{
  audio->end();
  delete audio;
}

void Main::setSignal(double a, int caller)
{ //function is called from two threads
  //For the test-signal
  //For the microphone
  // Figure out if we aked for the microphone or not...
  if (caller==SIGNAL_MICROPHONE && !needMicrophone) return;
  wavelets.signal(a,t++);
}

void Main::clear()
{
    wview->clear();
}

void Main::help()
{
    static QMessageBox* about = new QMessageBox( "WAVELET AUDIO LOGGER - Help",
      "<h3>wlog - A real-time wavelet audio logger</h3>"
      "<p>Set back while <tt>wlog</tt> decodes and visualizes your audio...</p>"
      "<p><i>Free for educational use - on4cko</i></p>"
    , QMessageBox::Information, 1, 0, 0, this, 0, FALSE );
    about->setButtonText( 1, "Dismiss" );
    about->show();
}


void Main::toggleDoubleBuffer()
{
    bool s = !options->isItemChecked(menuid_dbf);
    options->setItemChecked(menuid_dbf,s);
    canvas.setDoubleBuffering(s);
}

void Main::enlarge()
{
    canvas.resize(canvas.width()*4/3, canvas.height()*4/3);
    init();
}

void Main::shrink()
{
    canvas.resize(canvas.width()*3/4, canvas.height()*3/4);
    init();
}

void Main::resizeEvent ( QResizeEvent *e )
{
  canvas.resize(e->size().width(), e->size().height() );
  init();
}


void Main::menuEdit(int menuid)
{
  bool sel = edit->isItemChecked(menuid);
  int  i;
  
  if (menuid_s == menuid)        { edit->setItemChecked(menuid_s,true); edit->setItemChecked(menuid_d,false); goto end; }
  else if (menuid_d == menuid)   { edit->setItemChecked(menuid_s,false); edit->setItemChecked(menuid_d,true); goto end; }

  edit->setItemChecked(menuid,!sel);
  for ( i=0; i<MAX_OCTV; ++i) { if (edit->isItemChecked(menuid_o[i])) goto end; }

  //all components are switched of and we don't want that!
  // revert to octave 0

  edit->setItemChecked(menuid_o[ 0 ],true);

end:
  init();
}

void Main::menuSync(int menuid)
{
  bool sel = sync->isItemChecked(menuid);
  int  i,o;

  //handler for two menu-events:lock scale and sync octave  
  if (menuid_max == menuid)    {  
    //lock scale
    sync->setItemChecked(menuid_max,!sel); 
    wavelets.updateMax(!sel); 
  } else {
    //sync on octave X
    for (o=i=0; i<MAX_OCTV; ++i) { 
      if (menuid==menuid_sync[i]) o=i; 
      sync->setItemChecked(menuid_sync[i],false);
    }
    sync->setItemChecked(menuid,true);
    wavelets.setSync( o );
  }
}

int  Main::selectedS()
{ return edit->isItemChecked(menuid_s);
}

int  Main::selectedOctave(int i)
{ return edit->isItemChecked(menuid_o[i]);
}

int  Main::selectedMaxLock()
{ return sync->isItemChecked(menuid_max);
}

int  Main::selectedSource()
{ for (int i=0; i<SIGNAL_MAX; ++i)   {
      if (signal->isItemChecked(menuid_source[ i ]))  return i;
    }
  return SIGNAL_SIN;
}


void Main::menuSignal(int menuid)
{
  for (int i=0; i<SIGNAL_MAX; ++i) { 
    signal->setItemChecked(menuid_source[i],false);
  }
  signal->setItemChecked(menuid,true);
  needMicrophone = (menuid_source[ SIGNAL_MICROPHONE ]==menuid)?1:0;
}
