#ifndef WLOG_H
#define WLOG_H

#include <qpopupmenu.h>
#include <qmainwindow.h>
#include <qintdict.h>
#include <qcanvas.h>

#define DATAPOINTS 100

#define RTTI_TIMER 1000
#define RTTI_DATA  5000

#include "wavelets.h"
#include "audio.h"

const enum SignalSource {
  SIGNAL_SIN,  
  SIGNAL_SQUARE,
  SIGNAL_MICROPHONE,
  SIGNAL_MAX
};

class Audio;
class Main;
//ViewTimer inherits from QCanvasSprite to get a free/easy timer on our canvas
class ViewTimer : public QCanvasSprite {
  public:
    ViewTimer(QCanvas*, Main *);
    void        advance(int);
    virtual int rtti () const { return RTTI_TIMER; }
  protected:
    Main    *base;
};

class WaveletView : public QCanvasView {
    Q_OBJECT

public:
    WaveletView(QCanvas&, QWidget* parent=0, const char* name=0, WFlags f=0);
    void clear();

signals:
    void status(const QString&);
};

class Main : public QMainWindow {
    Q_OBJECT

public:
    Main(QCanvas&, QWidget* parent=0, const char* name=0, WFlags f=0);
    ~Main();
    Wavelets  *getWavelets()  { return &wavelets; }

    int    selectedS();
    int    selectedOctave(int);
    int    selectedMaxLock();
    int    selectedSource();

    void   setSignal(double , int);
    int    getTime() { return t; }
public slots:
    void help();

private slots:
    void clear();
    void init();
    void menuEdit(int);
    void menuSync(int);
    void menuSignal(int);

    void enlarge();
    void shrink();

    void toggleDoubleBuffer();

protected:
  void   resizeEvent ( QResizeEvent * );
  

private:
    QCanvas&         canvas;
    WaveletView     *wview;
    ViewTimer        timer;     

    QPopupMenu      *options,*edit,*sync,*signal;
    int              menuid_dbf;
    int              menuid_s,menuid_d;
    int              menuid_o[ MAX_OCTV ];
    int              menuid_max;
    int              menuid_sync[ MAX_OCTV ];
    int              menuid_source[ SIGNAL_MAX ];
    int              needMicrophone;  //shorcut to selectedSource()==MICROPHONE
    
private:
    Wavelets         wavelets;
    Audio           *audio;
    int              t;
};




#endif
