/*
 * Calculate wavelets
 *  This class accepts a signal in the 'signal' function
 *  and does the transformation.
 *  Readout happens via the 'get' function.
 *
 * OH - oct 2002
 */


#include "wavelets.h"
#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#define C0 0.4829629131445341
#define C1 0.8365163037378079 
#define C2 0.2241438680420134
#define C3 -0.1294095225512604

#define OUTOFBOUND 3.0    //avoid signals that are too large...
#define ALPHA      0.99   //adaptive scaling of signal

Wavelets::Wavelets(int n_) : n(n_), sync_on(0), lockedscale(0)
{
  ss = new double[n*MAX_OCTV];
  ds = new double[n*MAX_OCTV];

  if (!ss || !ds) FATAL("memory");
//  memset(max_data,0,sizeof(max_data)); 
  //memset(max_data_smooth,0,sizeof(max_data_smooth)); 
//  for (int i=0; i<MAX_OCTV; ++i) max_index[0][i]=max_index[1][i]=-1;
//  for ( i=0; i<MAX_OCTV; ++i) max_data_smooth[0][i]=max_data_smooth[1][i]=1.0;
  global_max[0]=global_max[1]=(1);
  local_max[0]=local_max[1]=(1);
  memset(x,0,sizeof(x));
}

Wavelets::~Wavelets()
{
  delete []ss;
  delete []ds;
}

void Wavelets::signal(double a,int t)
{
  double s,d,val;
  int    tick,i; //,k;

  for (int j=0; j<MAX_OCTV; ++j) {
      if (j==0)   shft_sd( x[j] , a,  &s ,&d , j, t,  &tick);
      else        shft_sd( x[j] , s , &s ,&d , j, t , &tick);

      if (tick && j==sync_on)   {
        /* scroll buffer one step*/
        memmove( ss , ss+MAX_OCTV, sizeof(double)*(n-1)*MAX_OCTV );
        memmove( ds , ds+MAX_OCTV, sizeof(double)*(n-1)*MAX_OCTV );
      }
      if (tick) {
        ss[ (n-1)*MAX_OCTV + j] = s;
        ds[ (n-1)*MAX_OCTV + j] = d;

        /* update maxima */
        for (i=0; !lockedscale && i<2; ++i) {
          val = fabs( (i==0)?s:d );

          if (global_max[i]<val) local_max[i]=val;
          global_max[i] = ALPHA*global_max[i] + (1-ALPHA)*local_max[i];
          local_max[i] =  ALPHA*local_max[i];
        } //for maximum (s or d)

      } //for all octaves
  }
}


double Wavelets::get(int i, int get_s, int o, int needGlobalMax)
{  double a,m;
  /* return the coef.as a number in range -1..1, if too large (due to smooth scaling)
   * return a nul */
  //m = (needGlobalMax)?global_max[get_s]:max_data_smooth[get_s][o];
  m = global_max[get_s];
  if (get_s==0) {
    a =  ss[(i)*MAX_OCTV+o]/m;
    return (fabs(a)>OUTOFBOUND)?0.0:a;
  } else {
    a =  ds[(i)*MAX_OCTV+o]/m;
    return (fabs(a)>OUTOFBOUND)?0.0:a;
  }
}



void Wavelets::shft_sd( double a[] , double s0,  double *s,double *d, int stage, int count, int *calculated)
{  
  if ( !( count&((1<<(1+stage))-1) )    ) {
    *s             = C0*a[0]+C1*a[1]+C2*a[2]+C3*a[3];
    *d             = C3*a[0]-C2*a[1]+C1*a[2]-C0*a[3];
    *calculated    = 1;
  } else {
    *calculated    = 0;
  }

  if ( !( count&((1<<(stage))-1) )    ) {  
    memmove(a,a+1,sizeof(double)*3);
    a[3] = s0;
  } 
}
